import os
import pygame
from pygame.locals import *
from consts import *
from game import g
from melee import Flamethrower

POSEUPDATES = [30, 50, 30, 20, 30, 20, 20]
FPS = 50
FONTSIZE = 36

LYRICSOFFSET = 0
GEMSOFFSET = 0

class Dancer(pygame.sprite.Sprite):
    def __init__(self, images, inrect):
        pygame.sprite.Sprite.__init__(self)

        self.rect = inrect
        self.images = images
        self.holdframe = 0
        self.currentimage = 0

    def update(self, time_passed):
        if self.holdframe > 0:
            self.holdframe -= 1
        if self.holdframe == 0:
            self.currentimage = 0
        self.image = self.images[self.currentimage]

    def dance(self, dancemove):
        self.holdframe = POSEUPDATES[dancemove - 1]
        self.currentimage = dancemove

class DDRGlobals(object):
    @classmethod
    def init_globals(cls):
        cls.g_currentgem = None
        cls.g_hitnotes = 0
        cls.g_missednotes = 0

rivalimages = (pygame.image.load(os.path.join("data","dancers","rival0.png")),
               pygame.image.load(os.path.join("data","dancers","rival1.png")),
               pygame.image.load(os.path.join("data","dancers","rival2.png")),
               pygame.image.load(os.path.join("data","dancers","rival3.png")),
               pygame.image.load(os.path.join("data","dancers","rival4.png")),
               pygame.image.load(os.path.join("data","dancers","rival5.png")),
               pygame.image.load(os.path.join("data","dancers","rival6.png")),
               pygame.image.load(os.path.join("data","dancers","rival7.png")) )
rivalrect = rivalimages[0].get_rect()
rivalrect.midbottom = (840, 283)
spaceimages = (pygame.image.load(os.path.join("data","dancers","space0.png")),
               pygame.image.load(os.path.join("data","dancers","space1.png")),
               pygame.image.load(os.path.join("data","dancers","space2.png")),
               pygame.image.load(os.path.join("data","dancers","space3.png")),
               pygame.image.load(os.path.join("data","dancers","space4.png")),
               pygame.image.load(os.path.join("data","dancers","space5.png")),
               pygame.image.load(os.path.join("data","dancers","space6.png")),
               pygame.image.load(os.path.join("data","dancers","space7.png")) )
spacelionrect = spaceimages[0].get_rect()
spacelionrect.midbottom = (520, 283)

g_rivaldancer = Dancer(rivalimages, rivalrect)
g_spaceliondancer = Dancer(spaceimages, spacelionrect)

gc_startaccepting = 250
gc_perfecthit = 0
gc_stopaccepting = -gc_startaccepting
gc_remove = -2000
gc_gemsbasey = 700
gc_gemheight = 50
gc_gemwidth = gc_gemheight
gc_msperpixel = gc_startaccepting / gc_gemwidth
gc_perfecthitx = 100

def timetox(time):
    return gc_perfecthitx + ((time - gc_startaccepting) / gc_msperpixel)

# gem colors:
COLOR_NORMAL = 0
COLOR_GRAY = 1
COLOR_RED = 2

# keys:
KEYS = (K_DOWN, K_LEFT, K_RIGHT, K_UP, K_a, K_s, K_d)
normalgems = ( pygame.image.load(os.path.join("data","gems","normal1.png")),
               pygame.image.load(os.path.join("data","gems","normal2.png")),
               pygame.image.load(os.path.join("data","gems","normal3.png")),
               pygame.image.load(os.path.join("data","gems","normal4.png")),
               pygame.image.load(os.path.join("data","gems","normal5.png")),
               pygame.image.load(os.path.join("data","gems","normal6.png")),
               pygame.image.load(os.path.join("data","gems","normal7.png")) )
graygems = ( pygame.image.load(os.path.join("data","gems","gray1.png")),
             pygame.image.load(os.path.join("data","gems","gray2.png")),
             pygame.image.load(os.path.join("data","gems","gray3.png")),
             pygame.image.load(os.path.join("data","gems","gray4.png")),
             pygame.image.load(os.path.join("data","gems","gray5.png")),
             pygame.image.load(os.path.join("data","gems","gray6.png")),
             pygame.image.load(os.path.join("data","gems","gray7.png")) )
redgems = ( pygame.image.load(os.path.join("data","gems","red1.png")),
            pygame.image.load(os.path.join("data","gems","red2.png")),
            pygame.image.load(os.path.join("data","gems","red3.png")),
            pygame.image.load(os.path.join("data","gems","red4.png")),
            pygame.image.load(os.path.join("data","gems","red5.png")),
            pygame.image.load(os.path.join("data","gems","red6.png")),
            pygame.image.load(os.path.join("data","gems","red7.png")) )


class Gem(pygame.sprite.Sprite):
    def __init__(self, timeleft, myexpectedkey):
        pygame.sprite.Sprite.__init__(self)

        self.startedaccepting = False
        self.perfecthitpassed = False
        self.stoppedaccepting = False
        self.colored = COLOR_NORMAL

        self.timeleft = timeleft + GEMSOFFSET
        self.myexpectedkey = myexpectedkey

    def update(self, time_passed):
        self.timeleft -= time_passed
        if self.timeleft < gc_startaccepting and not self.startedaccepting:
            DDRGlobals.g_currentgem = self
            self.startedaccepting = True
        if self.timeleft < gc_perfecthit and not self.perfecthitpassed:
            g_rivaldancer.dance(self.myexpectedkey)
            self.perfecthitpassed = True
        if self.timeleft < gc_stopaccepting and not self.stoppedaccepting:
            if DDRGlobals.g_currentgem == self:
                DDRGlobals.g_currentgem = None
            self.stoppedaccepting = True
            if self.colored == COLOR_NORMAL:
                self.colored = COLOR_RED
                DDRGlobals.g_missednotes += 1

        self.update_image_and_rect()

    def update_image_and_rect(self):
        y = gc_gemsbasey - (self.myexpectedkey * gc_gemheight)
        x = timetox(self.timeleft)
        if self.colored == COLOR_NORMAL:
            self.image = normalgems[self.myexpectedkey - 1]
        elif self.colored == COLOR_GRAY:
            self.image = graygems[self.myexpectedkey - 1]
        elif self.colored == COLOR_RED:
            self.image = redgems[self.myexpectedkey - 1]

        self.rect = self.image.get_rect().move(x, y)

    def attempthit(self, pressedkey):
        if pressedkey == self.myexpectedkey and self.colored == COLOR_NORMAL:
            DDRGlobals.g_hitnotes += 1
            self.colored = COLOR_GRAY
        else:
            DDRGlobals.g_missednotes += 1
            self.colored = COLOR_RED


def pump_events():
    for event in pygame.event.get():
        if event.type == KEYDOWN:
            expectedkey = 0
            if event.key == KEYS[0]:
                expectedkey = 1
            elif event.key == KEYS[1]:
                expectedkey = 2
            elif event.key == KEYS[2]:
                expectedkey = 3
            elif event.key == KEYS[3]:
                expectedkey = 4
            elif event.key == KEYS[4]:
                expectedkey = 5
            elif event.key == KEYS[5]:
                expectedkey = 6
            elif event.key == KEYS[6]:
                expectedkey = 7
            elif event.key == K_ESCAPE:
                return True

            if expectedkey != 0:
                if DDRGlobals.g_currentgem is None:
                    DDRGlobals.g_missednotes += 1
                else:
                    DDRGlobals.g_currentgem.attempthit(expectedkey)
                g_spaceliondancer.dance(expectedkey)
        elif event.type == QUIT:
            return True

gemdata = [(8728, 1), # Sax enters
    (13092, 2), (13092 + 548, 1), # Strings enter
    (16714, 1), (16714 + 274, 2), (16714 + 548, 3), (16714 + 822, 7),  #"You are my love"
    (18906, 1), (18906 + 548, 2), #"My space love"
    (21098, 1), (21098 + 274, 2), (21098 + 548, 3), (21098 + 822, 7),   #"You are my lion"
    (23290, 7), (23290 + 274, 6), (23290 + 548, 5), (23290 + 822, 1), #"I hear you roar"
    (25911, 2), # [Roar]
    (26304, 3), (26304 + 548, 5), (26304 + 822, 3), #"Everyone will terrifyingly run"
    (28496, 3), (28496 + 548, 5), #"When you are in town"
    (30687, 7), (30687 + 274, 7), #"No one can beat your belt"
    (33702, 5), (33702 + 274, 2), (33702 + 548, 4), #"Space Lion"
    (35060, 6), # [Crash]
    (35894, 5), (35894 + 274, 2), (35894 + 548, 4), #"Space Lion"
    (37263, 6), (37263 + 548, 5), (37263 + 822, 6), #"Fighting for the poor and loved"
    (40278, 5), (40278 + 274, 2), (40278 + 548, 4), #"Space Lion"
    (42196, 1), (42196 + 274, 2), (42196 + 548, 3), (42196 + 822, 7), #"Make the aliens cry"
    (44114, 1), (44114 + 548, 2), #"They deserve to die"
    (46305, 1), (46305 + 274, 2), (46305 + 548, 3), (46305 + 822, 5), #"No hero is greater"
    (48497, 4), #"Than my love"
    (51511, 3), (51511 + 548, 5), (51511 + 822, 3), #"Everyone will sing of your guts"
    (53703, 3), (53703 + 822, 4), #"No man is as brave as you"
    (55895, 5), #"You are an honest police officer"
    (58910, 5), (58910 + 274, 2), (58910 + 548, 4),  #"Space Lion"
    (60270, 7), # [Belt crack]
    (61102, 5), (61102 + 274, 2), (61102 + 548, 4),  #"Space Lion"
    (62471, 6), (62471 + 548, 5), (62471 + 822, 6), #"Fighting for the poor and Loved"
    (65470, 5), (65470 + 274, 2), (65470 + 548, 4),  #"Space Lion"
    (67678, 5), (67678 + 274, 2), (67678 + 548, 4),  #"Space Lion"
    (69870, 5), (69870 + 274, 2), (69870 + 548, 4),  #"Space Lion"
    (72472, 5), (72472 + 274, 4), (72472 + 548, 7),  #"Space Lion"
    (74520, 2), # [Roar]
]

g_gems = [Gem(*args) for args in gemdata]


class Lyric(pygame.sprite.Sprite):
    def __init__(self, timeleft, text, font):
        pygame.sprite.Sprite.__init__(self)

        self.font = font
        self.text = text
        self.image = self.font.render(self.text, False, (0, 127, 255))
        self.rect = self.image.get_rect()
        self.timeleft = timeleft + LYRICSOFFSET
        self.colorchanged = False

    def update(self, time_passed):
        self.timeleft -= time_passed
        self.rect.midleft = timetox(self.timeleft), 734
        if self.timeleft < 0 and not self.colorchanged:
            self.image = self.font.render(self.text, False, (255, 255, 0))



def init_g_lyrics():
    global g_lyrics

    lyricsfont = pygame.font.Font(None, FONTSIZE)
    lyricdata = [(5000, "To dance, press the keys as they cross the line!"),
        (16714, "You are my love"),
        (18906, "My space love"),
        (21098, "You are my lion"),
        (23290, "I hear you roar"),
        (26304, "Everyone will terrifyingly run"),
        (28496, "When you are in town"),
        (30687, "No one can beat your belt"),
        (33702, "Space Lion"),
        (35894, "Space Lion"),
        (37263, "Fighting for the poor and loved"),
        (40278, "Space Lion"),
        (42196, "Make the aliens cry"),
        (44114, "They deserve to die"),
        (46305, "No hero is greater"),
        (48497, "Than my love"),
        (51511, "Everyone will sing of your guts"),
        (53703, "No man is as brave as you"),
        (55895, "You are an honest police officer"),
        (58910, "Space Lion"),
        (61102, "Space Lion"),
        (62471, "Fighting for the poor and loved"),
        (65470, "Space Lion"),
        (67678, "Space Lion"),
        (69870, "Space Lion"),
        (72472, "Space Liooooooooon"),
    ]
    g_lyrics = [Lyric(*(args + (lyricsfont,))) for args in lyricdata]

def draw_background(screen, background):
    #screen.fill((0, 0, 0))
    screen.blit(background, (0, 0))

def game_loop(screen):
    start = pygame.time.get_ticks()

    spritegroup = pygame.sprite.OrderedUpdates()
    [spritegroup.add(g) for g in g_gems + g_lyrics]
    spritegroup.add(g_rivaldancer)
    spritegroup.add(g_spaceliondancer)

    t = pygame.time.get_ticks()
    clock = pygame.time.Clock()

    update_all = True

    background = pygame.image.load(os.path.join("data", "bg05.jpg"))

    while True:
        if pump_events():
            break

        new_t = pygame.time.get_ticks()
        spritegroup.update(new_t - t)
        t = new_t

        draw_background(screen, background)
        update_rects = spritegroup.draw(screen)

        if update_all:
            pygame.display.update()
        else:
            pygame.display.update(update_rects)

        if (pygame.time.get_ticks() - start) / 1000. > 84: # length of song
            return

        clock.tick(FPS)

class DDRLevel(object):
    LEVEL_ASSET = "level5"

    def play(self):
        if g.assets.has_key("level-voice"):
            snd = g.assets["level-voice"]["sound"]
            snd.play()
            
        DDRGlobals.init_globals()
        init_g_lyrics()
        g.score.pickup_weapon(Flamethrower)
        game_loop(g.screen)
        self.result = LEVEL_BEATEN

#def main():
#    pygame.init()
#    DDRGlobals.init_globals()
#    screen = pygame.display.set_mode((1366, 768), 0, 32)
#    pygame.mouse.set_visible(False)
#
#    init_g_lyrics()
#
#    music = pygame.mixer.Sound("data", "spaceliontheme.ogg")
#    music.play()
#
#    game_loop(screen)
#
#if __name__ == '__main__':
#    main()
