from locals import *

class IntroScreenBase(object):
    INTRO_SOUND = None
    INTRO_MUSIC = None
    MUSIC_VOLUME = 0.5
    SOUND_VOLUME = 1.0

    def __init__(self, screen):
        self.screen = screen

    def _img(self, fn):
        return pygame.image.load(fn).convert_alpha()

    def play(self):
        channels = []

        if self.INTRO_SOUND is not None:
            snd = pygame.mixer.Sound(self.INTRO_SOUND)
            snd.set_volume(self.SOUND_VOLUME)
            channels.append(snd.play())

        if self.INTRO_MUSIC is not None:
            mus = pygame.mixer.Sound(self.INTRO_MUSIC)
            mus.set_volume(self.MUSIC_VOLUME)
            channels.append(mus.play(loops=-1))

        self.screen.fill((0, 0, 0))
        self.screen.blit(self._img(self.INTRO_IMAGE), (0, 0))
        pygame.display.update()

        while True:
            for event in pygame.event.get():
                if event.type == QUIT or (event.type == KEYUP and event.key == K_ESCAPE):
                    for c in channels:
                        c.stop()
                    return USER_EXIT

                elif event.type == KEYDOWN and event.key == K_SPACE:
                    for c in channels:
                        c.stop()
                    return LEVEL_BEATEN

class TitleScreen(IntroScreenBase):
    INTRO_IMAGE = os.path.join("data", "title.png")

class IntroScreen(IntroScreenBase):
    INTRO_IMAGE = os.path.join("data", "intro.png")
    INTRO_MUSIC = os.path.join("data", "intro.ogg")
    INTRO_SOUND = os.path.join("data", "voices", "intro_sequence.ogg")
    MUSIC_VOLUME = 0.1

class GameOverScreen(IntroScreenBase):
    INTRO_IMAGE = os.path.join("data", "gameover.png")
    INTRO_SOUND = os.path.join("data", "gameover.ogg")
    SOUND_VOLUME = 0.5

class EndingScreen(IntroScreenBase):
    INTRO_IMAGE = os.path.join("data", "ending.png")
    INTRO_SOUND = os.path.join("data", "voices", "end_sequence2-censored.ogg")
    INTRO_MUSIC = os.path.join("data", "music", "intermission.ogg")

    MUSIC_VOLUME = 0.1

class CreditsScreen(IntroScreenBase):
    INTRO_IMAGE = os.path.join("data", "credits.png")
    INTRO_MUSIC = os.path.join("data", "music", "outerspaceboss.ogg")
    INTRO_SOUND = os.path.join("data", "voices", "boss_battle.ogg")

    MUSIC_VOLUME = 0.4
